



// print debug messages or not to serial
const boolean SerialDisplay = false;
// pins assignments
const int photocellPin = A0;                 // photocell connected to analog 0
const int enableCoopDoorMotor = 7;          // enable motor b - pin 7
const int directionCloseCoopDoor = 8;  // direction close motor b - pin 8
const int directionOpenCoopDoor = 9;   // direction open motor b - pin 9
const int bottomSwitchPin = 13;              // bottom switch is connected to pin 13
const int topSwitchPin = 12;                 // top switch is connected to pin 12
const int coopDoorOpenLed = 2;              // led to signal open 2
const int coopDoorClosedLed = 4;            // led to signal closed 4



// variables



// photocell
int photocellReading;                            // analog reading of the photocel
int photocellReadingLevel;                       // photocel reading levels (dark, light)

//top and bottom switches of coop door

// top switch

int topSwitchPinVal;                   // top switch var for reading the pin status
int topSwitchPinVal2;                  // top switch var for reading the pin delay/debounce status
int topSwitchState;                    // top switch var for to hold the switch state

// bottom switch

int bottomSwitchPinVal;                // bottom switch var for reading the pin status
int bottomSwitchPinVal2;               // bottom switch var for reading the pin delay/debounce status
int bottomSwitchState;                 // bottom switch var for to hold the switch state



// photocell reading delay
unsigned long lastPhotocellReadingTime = 0;
unsigned long photocellReadingDelay = 6000;   // 10 minutes

// debounce delay
unsigned long lastDebounceTime = 0;
unsigned long debounceDelay = 100;





// ************************************** the setup **************************************

void setup(void) {

  Serial.begin(9600); // initialize serial port hardware



  // coop door

  // coop door motor
  pinMode (enableCoopDoorMotor, OUTPUT);           // enable motor pin = output
  pinMode (directionCloseCoopDoor, OUTPUT);   // motor close direction pin = output
  pinMode (directionOpenCoopDoor, OUTPUT);    // motor open direction pin = output

  // coop door leds
  pinMode (coopDoorOpenLed, OUTPUT);                // enable coopDoorOpenLed = output
  pinMode (coopDoorClosedLed, OUTPUT);              // enable coopDoorClosedLed = output
  digitalWrite(coopDoorClosedLed, LOW);

  // coop door switches
  // bottom switch
  pinMode(bottomSwitchPin, INPUT);                  // set bottom switch pin as input
  digitalWrite(bottomSwitchPin, HIGH);              // activate bottom switch resistor

  // top switch
  pinMode(topSwitchPin, INPUT);                     // set top switch pin as input
  digitalWrite(topSwitchPin, HIGH);                 // activate top switch resistor



}

// ************************************** functions **************************************



// operate the coop door

// photocel to read levels of exterior light

void ReadPhotoCell() { // function to be called repeatedly - per coopPhotoCellTimer set in setup

  photocellReading = analogRead(photocellPin);

  if ((unsigned long)(millis() - lastPhotocellReadingTime) >= photocellReadingDelay) {
    lastPhotocellReadingTime = millis();

    //  set photocel threshholds
    //  if (photocellReading >= 0 && photocellReading <= 5) { photocellReadingLevel = '1'; if (SerialDisplay) { Serial.println(" Photocel Reading Level:"); Serial.println(" - Dark"); } } else if (photocellReading >= 6 && photocellReading <= 120) { photocellReadingLevel = '2'; if (SerialDisplay) { Serial.println(" Photocel Reading Level:"); Serial.println(" - Twilight"); } } else if (photocellReading >= 125 ) {
    if (photocellReading <= 50) {
      photocellReadingLevel = '1';
    }
    else if (photocellReading >= 150 )
    {

      photocellReadingLevel = '2';
//      Serial.println(" Photocel Reading Level:");
//      Serial.println(" - Light");

    }

    Serial.println(" Photocel Reading = ");
    Serial.println(photocellReading);

  }
}

//debounce bottom switch

void debounceBottomSwitch() {

  //debounce bottom switch
  bottomSwitchPinVal = digitalRead(bottomSwitchPin);       // read input value and store it in val

  if ((unsigned long)(millis() - lastDebounceTime) > debounceDelay) {    // delay 10ms for consistent readings

    bottomSwitchPinVal2 = digitalRead(bottomSwitchPin);    // read input value again to check or bounce

    if (bottomSwitchPinVal == bottomSwitchPinVal2) {       // make sure we have 2 consistant readings
      if (bottomSwitchPinVal != bottomSwitchState) {       // the switch state has changed!
        bottomSwitchState = bottomSwitchPinVal;
      }
    }
  }
}



// debounce top switch
void debounceTopSwitch() {

  topSwitchPinVal = digitalRead(topSwitchPin);             // read input value and store it in val

  if ((unsigned long)(millis() - lastDebounceTime) > debounceDelay) {     // delay 10ms for consistent readings

    topSwitchPinVal2 = digitalRead(topSwitchPin);          // read input value again to check or bounce

    if (topSwitchPinVal == topSwitchPinVal2) {             // make sure we have 2 consistant readings
      if (topSwitchPinVal != topSwitchState) {             // the button state has changed!
        topSwitchState = topSwitchPinVal;
      }
    }
  }
}


// stop the coop door motor
void stopCoopDoor() {
  digitalWrite (directionCloseCoopDoor, LOW);      // turn off motor close direction
  digitalWrite (directionOpenCoopDoor, LOW);       // turn on motor open direction
  analogWrite (enableCoopDoorMotor, 0);                 // enable motor, 0 speed
}



// close the coop door motor (motor dir close = clockwise)
void CloseCoopDoor() {
  digitalWrite (directionCloseCoopDoor, HIGH);     // turn on motor close direction
  digitalWrite (directionOpenCoopDoor, LOW);       // turn off motor open direction
  analogWrite (enableCoopDoorMotor, 255);               // enable motor, full speed
  if (bottomSwitchPinVal == 0) {                         // if bottom switch circuit is closed
    stopCoopDoor();
    digitalWrite(coopDoorOpenLed, LOW);
    digitalWrite(coopDoorClosedLed, HIGH);

    Serial.println(" Coop Door Closed");

  }
}



// open the coop door (motor dir open = counter-clockwise)
void OpenCoopDoor() {
  digitalWrite(directionCloseCoopDoor, LOW);       // turn off motor close direction
  digitalWrite(directionOpenCoopDoor, HIGH);       // turn on motor open direction
  analogWrite(enableCoopDoorMotor, 255);                // enable motor, full speed
  if (topSwitchPinVal == 0) {                            // if top switch circuit is closed
    stopCoopDoor();
    digitalWrite(coopDoorOpenLed, HIGH);
    digitalWrite(coopDoorClosedLed, LOW);

    Serial.println(" Coop Door open");

  }
}





// do the coop door
void doCoopDoor() {
  if (photocellReadingLevel  == '1') {            // if its dark
    debounceTopSwitch();                    // read and debounce the switches
    debounceBottomSwitch();
    CloseCoopDoor();                      // close the door
  }
  if (photocellReadingLevel  == '2')               // if its light
  {
    debounceTopSwitch();                    // read and debounce the switches
    debounceBottomSwitch();
    OpenCoopDoor();                       // Open the door
  }
}

// ************************************** the loop **************************************

void loop() {

  ReadPhotoCell();
  doCoopDoor();

}
